﻿using Infragistics;
using Infragistics.Controls;
using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows;

namespace WpfApplication2
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = ChartData.GenerateData(10);
        }       
    }

    public class ZoomBarRangeBehaviour : Behavior<XamDataChart>
    {

        private XamDataChart Chart
        {
            get
            {
                return AssociatedObject as XamDataChart;
            }
        }

        public XamZoombar HorizontalZoombar
        {
            get { return (XamZoombar)GetValue(HorizontalZoombarProperty); }
            set { SetValue(HorizontalZoombarProperty, value); }
        }

        // Using a DependencyProperty as the backing store for HorizontalZoombar. This enables animation, styling, binding, etc...
        public static readonly DependencyProperty HorizontalZoombarProperty =
        DependencyProperty.Register("HorizontalZoombar", typeof(XamZoombar), typeof(ZoomBarRangeBehaviour), new PropertyMetadata(SetZoombar));


        private static void SetZoombar(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            Binding binding = new Binding
            {
                Source = (obj as ZoomBarRangeBehaviour).Chart,
                Path = new PropertyPath("HorizontalZoombar.Range"),
                Mode = BindingMode.TwoWay
            };
            (obj as ZoomBarRangeBehaviour).HorizontalZoombar.SetBinding(XamZoombar.RangeProperty, binding);
            (obj as ZoomBarRangeBehaviour).HorizontalZoombar.Range = new Range { Minimum = 0.9, Maximum = 1 };

            // detach the old events so we don't leak memory
            (obj as ZoomBarRangeBehaviour).DetachEvents();

            // attach the new events
            (obj as ZoomBarRangeBehaviour).AttachEvents();


        }

        private void AttachEvents()
        {
            this.HorizontalZoombar.ZoomChanged += OnXamZoombarZoomChanged;
            //this.HorizontalZoombar.ZoomChanging+=HorizontalZoombar_ZoomChanging;
            if (this.Chart != null)
                this.Chart.WindowRectChanged += OnXamDataChartWindowRectChanged;
        }
        private void DetachEvents()
        {
            this.HorizontalZoombar.ZoomChanged -= OnXamZoombarZoomChanged;
            //this.HorizontalZoombar.ZoomChanging -= HorizontalZoombar_ZoomChanging;
            if (this.Chart != null)
                this.Chart.WindowRectChanged -= OnXamDataChartWindowRectChanged;
        }

        bool fromMouseWheel = false;
        double scaleH = -1;
        double scaleV = -1;
        double positionH = -1;
        double positionV = -1;
        public bool IsZoombarChanging;
        private void OnXamZoombarZoomChanged(object sender, ZoomChangedEventArgs e)
        {
            if (this.Chart == null) return;
            if (IsZoombarChanging) return;
            try
            {
                IsZoombarChanging = true;
                Range zoombarRange = e.NewRange;
                HorizontalZoombar.Range = zoombarRange;
                this.Chart.WindowRect = new Rect
                {
                    Y = 0,
                    Height = 1,
                    X = zoombarRange.Minimum,
                    Width = (zoombarRange.Maximum - zoombarRange.Minimum)
                };

            }
            finally
            {
                IsZoombarChanging = false;
            }
        }

        private void OnXamChart_PreviewMouseWheel(object sender, MouseWheelEventArgs e)
        {
            fromMouseWheel = true;
            scaleH = this.Chart.WindowScaleHorizontal;
            scaleV = this.Chart.WindowScaleVertical;
            positionH = this.Chart.WindowPositionHorizontal;
            positionV = this.Chart.WindowPositionVertical;
        }


        private void OnXamDataChartWindowRectChanged(object sender, RectChangedEventArgs e)
        {
            if (IsZoombarChanging) return;
            if (fromMouseWheel) return;
            try
            {
                IsZoombarChanging = true;
                Range newZoombarRange = new Range
                {
                    Minimum = e.NewRect.X,
                    Maximum = e.NewRect.X + e.NewRect.Width
                };
                if (this.HorizontalZoombar != null)
                {
                    this.HorizontalZoombar.Range = newZoombarRange;
                }
            }
            finally
            {
                IsZoombarChanging = false;
                fromMouseWheel = false;
            }
        }


        protected override void OnAttached()
        {
            base.OnAttached();
        }

    }






    public class ChartData
    {
        public DateTime Date { get; set; }
        public double Value1 { get; set; }
        public double Value2 { get; set; }
        public double Value3 { get; set; }

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Value1 = r.Next(20, 40),
                Value2 = r.Next(20, 40),
                Value3 = r.Next(20, 40),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = data[i - 1].Value1 + r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 + r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = data[i - 1].Value1 - r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 - r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }
}
